__version__ = "0.1.1"

from upstash_search.asyncio.index import AsyncIndex
from upstash_search.asyncio.search import AsyncSearch
from upstash_search.index import Index
from upstash_search.search import Search

__all__ = ["Search", "Index", "AsyncSearch", "AsyncIndex"]
