LIST_INDEXES_PATH = "/list-namespaces"
DELETE_INDEX_PATH = "/delete-namespace"
INFO_PATH = "/info"
UPSERT_PATH = "/upsert-data"
SEARCH_PATH = "/search"
FETCH_PATH = "/fetch"
DELETE_PATH = "/delete"
RANGE_PATH = "/range"
RESET_PATH = "/reset"
